#####################
## Create Figure 1 ##
#####################

mu = function(x){ log(1 + exp(6 * x)) }

# set parameters
n = 500; p = 20; sigma = sqrt(20)
X = matrix(runif(n * p, -1, 1), nrow = n)

# order X[,1] for easy visualization 
X[,1] = sort(X[,1])
truth = mu(X[,1])
Y = truth + sigma * rnorm(n)

# random forest predictions
forest = regression_forest(X, Y, tune.parameters = tune)
preds = predict(forest, estimate.variance = TRUE)

# construct dataframe with predictions and confidence intervals
df = data.frame(ticks = X[,1], predictions = preds$predictions,
                upper = preds$predictions + 1.96*sqrt(preds$variance.estimates),
                lower = preds$predictions - 1.96*sqrt(preds$variance.estimates))

ggplot(df, aes(ticks)) +
  geom_point(aes(y = predictions), color = "red", show.legend = F, cex = 0.6) +
  geom_line(aes(y = truth)) +
  geom_line(aes(y = upper),color = "gray",lty = 2) +
  geom_line(aes(y = lower),color = "gray",lty = 2) +
  xlab("x") + ylab("y") + theme_bw()
ggsave("results/rf_bias.jpeg")

# local linear forest training
ll.forest = ll_regression_forest(X, Y, enable.ll.split = TRUE, tune.parameters = tune)

# lasso to select local linear correction variables
lasso.mod = cv.glmnet(X, Y, alpha=1)
selected = as.numeric(predict(lasso.mod, type = "nonzero"))

# local linear forest prediction
preds.llf = predict(ll.forest, linear.correction.variables = selected, estimate.variance = TRUE)

# construct dataframe with predictions and confidence intervals
df = data.frame(ticks = X[,1], predictions = preds.llf$predictions,
                upper = preds.llf$predictions + 1.96*sqrt(preds.llf$variance.estimates),
                lower = preds.llf$predictions - 1.96*sqrt(preds.llf$variance.estimates))

ggplot(df, aes(ticks)) +
  geom_point(aes(y = predictions), color = "red", show.legend = F, cex = 0.6) +
  geom_line(aes(y = truth)) +
  geom_line(aes(y = upper), color = "gray", lty = 2) +
  geom_line(aes(y = lower), color = "gray", lty = 2) +
  xlab("x") + ylab("y") + theme_bw()
ggsave("results/llf_bias.jpeg")
